from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AppServiceInstanceMinimum(BaseResourceCheck):
    def __init__(self) -> None:
        # "App Services Plans provides a configurable number of instances that will run apps.
        # When a single instance is configured your app may be temporarily unavailable during unplanned interruptions.
        # In most circumstances, Azure will self-heal faulty app service instances automatically.
        # How-ever during this time there may interruptions to your workload."
        name = "Ensure App Service has a minimum number of instances for failover"
        id = "CKV_AZURE_212"
        supported_resources = ("azurerm_service_plan",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        worker_count = conf.get("worker_count")
        if worker_count and isinstance(worker_count, list):
            if worker_count[0] > 1:
                return CheckResult.PASSED
        return CheckResult.FAILED


check = AppServiceInstanceMinimum()



from abc import abstractmethod

from checkov.common.checks.base_check import BaseCheck
from checkov.serverless.checks.layer.registry import layer_registry


class BaseLayerCheck(BaseCheck):
    def __init__(self, name, id, categories, supported_entities, guideline=None):
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities,
                         block_type="serverless", guideline=guideline)
        layer_registry.register(self)

    def scan_entity_conf(self, conf, entity_type):
        return self.scan_layer_conf(conf)

    @abstractmethod
    def scan_layer_conf(self, conf):
        raise NotImplementedError()



from __future__ import annotations

import logging
from typing import Optional, List, Tuple, Dict, Any

from termcolor import colored

from checkov.secrets.consts import ValidationStatus

from checkov.common.bridgecrew.severities import Severity

from checkov.common.output.record import Record
from checkov.common.typing import _CheckResult

WARNING_SIGN_UNICODE = '\u26a0'
TEXT_BY_SECRET_VALIDATION_STATUS = {
    ValidationStatus.VALID.value: colored(f'\t{WARNING_SIGN_UNICODE} This secret has been validated'
                                          f' and should be prioritized\n', "red"),
    ValidationStatus.INVALID.value: '\tThis is not a valid secret and can be de-prioritized\n',
    ValidationStatus.UNKNOWN.value: '\tWe were not able to validate this secret\n',
    ValidationStatus.UNAVAILABLE.value: ''
}


class SecretsRecord(Record):
    def __init__(self,
                 check_id: str,
                 check_name: str,
                 check_result: _CheckResult,
                 code_block: List[Tuple[int, str]],
                 file_path: str,
                 file_line_range: List[int],
                 resource: str,
                 evaluations: Optional[Dict[str, Any]],
                 check_class: str,
                 file_abs_path: str,
                 entity_tags: Optional[Dict[str, str]] = None,
                 caller_file_path: Optional[str] = None,
                 caller_file_line_range: Optional[Tuple[int, int]] = None,
                 bc_check_id: Optional[str] = None,
                 resource_address: Optional[str] = None,
                 severity: Optional[Severity] = None,
                 bc_category: Optional[str] = None,
                 benchmarks: dict[str, list[str]] | None = None,
                 description: Optional[str] = None,
                 short_description: Optional[str] = None,
                 vulnerability_details: Optional[Dict[str, Any]] = None,
                 connected_node: Optional[Dict[str, Any]] = None,
                 details: Optional[List[str]] = None,
                 check_len: int | None = None,
                 definition_context_file_path: Optional[str] = None,
                 validation_status: Optional[str] = None
                 ):
        super().__init__(check_id=check_id,
                         check_name=check_name,
                         check_result=check_result,
                         code_block=code_block,
                         file_path=file_path,
                         file_line_range=file_line_range,
                         resource=resource,
                         evaluations=evaluations,
                         check_class=check_class,
                         file_abs_path=file_abs_path,
                         entity_tags=entity_tags,
                         bc_check_id=bc_check_id,
                         severity=severity,
                         details=details,
                         caller_file_path=caller_file_path,
                         caller_file_line_range=caller_file_line_range,
                         resource_address=resource_address,
                         bc_category=bc_category,
                         benchmarks=benchmarks,
                         description=description,
                         short_description=short_description,
                         vulnerability_details=vulnerability_details,
                         connected_node=connected_node,
                         check_len=check_len,
                         definition_context_file_path=definition_context_file_path
                         )
        self.validation_status = validation_status

    def _get_secret_validation_status_message(self) -> str:
        message = None
        if self.validation_status:
            message = TEXT_BY_SECRET_VALIDATION_STATUS.get(self.validation_status)

            if not message and self.validation_status != ValidationStatus.UNAVAILABLE.value:
                logging.debug(f'Got empty message for secret validation status = {self.validation_status}')

        return message or ''



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.util.type_forcers import force_int


class PasswordPolicyMaxLogin(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Ram Account Password Policy Max Login Attempts not > 5"
        id = "CKV_ALI_23"
        supported_resources = ['alicloud_ram_account_password_policy']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return 'max_login_attempts'

    def get_expected_value(self):
        return 3

    def scan_resource_conf(self, conf):
        """
            validates ram password policy
            https://registry.terraform.io/providers/aliyun/alicloud/latest/docs/resources/ram_account_password_policy
        :param conf: alicloud_ram_account_password_policy configuration
        :return: <CheckResult>
        """

        if conf.get('max_login_attempts'):
            length = force_int(conf.get('max_login_attempts')[0])
            if length is None:
                return CheckResult.UNKNOWN
            if length <= 5:
                return CheckResult.PASSED
            self.evaluated_keys = ["max_login_attempts"]
            return CheckResult.FAILED
        return CheckResult.PASSED


check = PasswordPolicyMaxLogin()



from __future__ import annotations
from abc import abstractmethod

from checkov.kubernetes.graph_builder.graph_components.blocks import KubernetesBlock


class K8SEdgeBuilder:

    @staticmethod
    @abstractmethod
    def should_search_for_edges(vertex: KubernetesBlock) -> bool:
        """
        implementation should examine vertex's attributes and indicate if it's potentially
        suitable for the concrete class's edge type.
        e.g: search for a label attribute in LabelSelectorEdgeBuilder's implementation
        """
        raise NotImplementedError

    @staticmethod
    @abstractmethod
    def find_connections(vertex: KubernetesBlock, vertices: list[KubernetesBlock]) -> list[int]:
        """
        implementation should search in each of the vertices for a possible connection
        to the vertex param according to the concrete class's rule(s).
        returns a list of the connected vertices' indices
        e.g: find vertices with a label attribute that match current vertex's selector attribute
        """
        raise NotImplementedError



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.common.util.data_structures_utils import find_in_dict
from checkov.common.util.type_forcers import force_list, extract_json
from checkov.terraform.image_referencer.base_provider import BaseTerraformProvider

if TYPE_CHECKING:
    from networkx import DiGraph


class AwsTerraformProvider(BaseTerraformProvider):
    def __init__(self, graph_connector: DiGraph) -> None:
        super().__init__(
            graph_connector=graph_connector,
            supported_resource_types=SUPPORTED_AWS_IMAGE_RESOURCE_TYPES,
        )


def extract_images_from_aws_apprunner_service(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    image_repo = find_in_dict(input_dict=resource, key_path="source_configuration/image_repository")
    if isinstance(image_repo, dict):
        repo_type = image_repo.get("image_repository_type")
        name = image_repo.get("image_identifier")
        if name and isinstance(name, str) and repo_type == "ECR_PUBLIC":
            image_names.append(name)

    return image_names


def extract_images_from_aws_batch_job_definition(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    properties = extract_json(resource.get("container_properties"))
    if isinstance(properties, dict):
        name = properties.get("image")
        if name and isinstance(name, str):
            image_names.append(name)

    # node properties are not supported yet
    # https://github.com/hashicorp/terraform-provider-aws/issues/20983

    return image_names


def extract_images_from_aws_codebuild_project(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    name = find_in_dict(input_dict=resource, key_path="environment/image")
    if name and isinstance(name, str):
        # AWS provided images have an internal identifier
        if not name.startswith("aws/codebuild/"):
            image_names.append(name)

    return image_names


def extract_images_from_aws_ecs_task_definition(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    definitions = extract_json(resource.get("container_definitions"))
    if isinstance(definitions, list):
        for definition in definitions:
            if isinstance(definition, dict):
                name = definition.get("image")
                if name and isinstance(name, str):
                    image_names.append(name)

    return image_names


def extract_images_from_aws_lightsail_container_service_deployment_version(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    containers = resource.get("container")
    if containers:
        for container in force_list(containers):
            if isinstance(container, dict):
                name = container.get("image")
                if name and isinstance(name, str):
                    image_names.append(name)

    return image_names


# needs to be at the bottom to add the defined functions
SUPPORTED_AWS_IMAGE_RESOURCE_TYPES = {
    "aws_apprunner_service": extract_images_from_aws_apprunner_service,
    "aws_batch_job_definition": extract_images_from_aws_batch_job_definition,
    "aws_codebuild_project": extract_images_from_aws_codebuild_project,
    "aws_ecs_task_definition": extract_images_from_aws_ecs_task_definition,
    "aws_lightsail_container_service_deployment_version": extract_images_from_aws_lightsail_container_service_deployment_version,
}



from __future__ import annotations

from typing import Any

from checkov.azure_pipelines.checks.base_azure_pipelines_check import BaseAzurePipelinesCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.yaml_doc.enums import BlockType


class DetectImageUsage(BaseAzurePipelinesCheck):
    def __init__(self) -> None:
        name = "Detecting image usages in azure pipelines workflows"
        id = "CKV_AZUREPIPELINES_5"
        super().__init__(
            name=name,
            id=id,
            categories=(CheckCategories.SUPPLY_CHAIN,),
            supported_entities=("jobs[]", "*.container[]"),
            block_type=BlockType.ARRAY,
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        return CheckResult.PASSED, conf


check = DetectImageUsage()



from checkov.terraform.module_loading.loaders.git_loader import GenericGitLoader
from checkov.terraform.module_loading.module_params import ModuleParams


class BitbucketLoader(GenericGitLoader):
    def discover(self, module_params: ModuleParams):
        self.module_source_prefix = "bitbucket.org"


loader = BitbucketLoader()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ACRAnonymousPullDisabled(BaseResourceCheck):
    ANONYMOUS_PULL_SKUS = {"Standard", "Premium"}  # noqa: CCE003  # a static attribute

    def __init__(self):
        name = "Ensures that ACR disables anonymous pulling of images"
        id = "CKV_AZURE_138"
        supported_resources = ("azurerm_container_registry",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # anonymous_pull_enabled only applies to Standard and Premium skus, by default is set to false
        if (
            "sku" in conf.keys()
            and isinstance(conf["sku"][0], str)
            and conf["sku"][0] in ACRAnonymousPullDisabled.ANONYMOUS_PULL_SKUS
            and "anonymous_pull_enabled" in conf.keys()
            and conf["anonymous_pull_enabled"][0]
        ):
            return CheckResult.FAILED

        return CheckResult.PASSED


check = ACRAnonymousPullDisabled()



from __future__ import annotations

from typing import TYPE_CHECKING, TypeVar

from igraph import Graph

from checkov.common.graph.db_connectors.db_connector import DBConnector
from checkov.common.graph.graph_builder import CustomAttributes

if TYPE_CHECKING:
    from checkov.common.graph.graph_builder.graph_components.blocks import Block  # noqa
    from checkov.common.graph.graph_builder.local_graph import LocalGraph

_Block = TypeVar("_Block", bound="Block")


class IgraphConnector(DBConnector[Graph]):
    def __init__(self) -> None:
        self.graph = Graph(directed=True)

    def save_graph(self, local_graph: LocalGraph[_Block], add_bulk_edges: bool = False) -> Graph:
        return self.networkit_from_local_graph(local_graph)

    def get_reader_endpoint(self) -> Graph:
        return self.graph

    def get_writer_endpoint(self) -> Graph:
        return self.graph

    def networkit_from_local_graph(self, local_graph: LocalGraph[_Block]) -> Graph:
        # colors = {
        #     "Pod": "blue",
        #     "Deployment": "magenta",
        #     "Service": "olive",
        # }
        #
        # random_colors = ("red", "blue", "olive", "gold", "magenta", "purple", "black")
        # random_colors_len = len(random_colors)
        # randrange(len(random_colors))

        for vertex in local_graph.vertices:
            attr = vertex.get_attribute_dict()
            self.graph.add_vertex(
                name=attr[CustomAttributes.HASH],
                block_type_=vertex.block_type,
                resource_type=attr[CustomAttributes.RESOURCE_TYPE] if CustomAttributes.RESOURCE_TYPE in attr else None,
                # label=attr[CustomAttributes.BLOCK_NAME],
                # color=colors.get(attr["kind"], "red"),
                attr=attr,
            )

        edges_to_add = [
            (
                e.origin,
                e.dest,

            )
            for e in local_graph.edges
        ]
        # edge_colors = [random_colors[randrange(random_colors_len)] for _ in range(len(local_graph.edges))]
        edge_attributes = {
            "label": [e.label for e in local_graph.edges],
            # "color": edge_colors,
            # "label_color": edge_colors,
        }
        self.graph.add_edges(edges_to_add, edge_attributes)

        # plot(self.graph, target='myfile.png', bbox=(0, 0, 2000, 2000), margin=100, vertex_label_dist=1, layout="circle")
        return self.graph

    def disconnect(self) -> None:
        # not used, but is an abstractmethod
        return None



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any


class KMSKeyRotationIsEnabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure KMS Key Rotation is enabled"
        id = "CKV_ALI_27"
        supported_resources = ['alicloud_kms_key']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self) -> str:
        return "automatic_rotation"

    def get_expected_value(self) -> Any:
        return "Enabled"


check = KMSKeyRotationIsEnabled()



from __future__ import annotations

import logging
import os
from collections.abc import Iterable
from pathlib import Path
from typing import TYPE_CHECKING, Callable

from checkov.common.runners.base_runner import filter_ignored_paths
from checkov.common.util.dockerfile import is_docker_file
from checkov.dockerfile.parser import parse

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction  # only in extra_stubs
    from typing_extensions import Literal

DOCKERFILE_STARTLINE: Literal["startline"] = "startline"
DOCKERFILE_ENDLINE: Literal["endline"] = "endline"


def get_scannable_file_paths(
    root_folder: str | Path | None = None, excluded_paths: list[str] | None = None
) -> set[str]:
    """Finds Dockerfiles"""

    file_paths: "set[str]" = set()
    if not root_folder:
        return file_paths

    for root, d_names, f_names in os.walk(root_folder):
        filter_ignored_paths(root, d_names, excluded_paths)
        filter_ignored_paths(root, f_names, excluded_paths)
        for file in f_names:
            if is_docker_file(file):
                file_path = os.path.join(root, file)
                file_paths.add(file_path)

    return file_paths


def get_files_definitions(
    files: Iterable[str], filepath_fn: Callable[[str], str] | None = None
) -> tuple[dict[str, dict[str, list[_Instruction]]], dict[str, list[str]]]:
    """Parses Dockerfiles into its definitions and raw data"""

    definitions = {}
    definitions_raw = {}

    for file in files:
        try:
            result = parse(file)

            path = filepath_fn(file) if filepath_fn else file
            definitions[path], definitions_raw[path] = result
        except TypeError:
            logging.info(f"Dockerfile skipping {file} as it is not a valid dockerfile template")
        except UnicodeDecodeError:
            logging.info(f"Dockerfile skipping {file} as it can't be read as text file")

    return definitions, definitions_raw


def get_abs_path(root_folder: str | None, file_path: str) -> str:
    """Creates the abs path

    There are a few cases here. If -f was used, there could be a leading / because it's an absolute path,
    or there will be no leading slash; root_folder will always be none.
    If -d is used, root_folder will be the value given, and -f will start with a / (hardcoded above).
    The goal here is simply to get a valid path to the file (which docker_file_path does not always give).
    """

    if root_folder and file_path.startswith("/"):
        # remove the leading slash, if it exists
        file_path = file_path[1:]

    path_to_convert = os.path.join(root_folder, file_path) if root_folder else file_path

    return os.path.abspath(path_to_convert)



from __future__ import annotations

import json
import logging
from abc import abstractmethod
from collections.abc import Iterable
from typing import TYPE_CHECKING, Any

from checkov.ansible.checks.registry import registry
from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckResult

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckCategories


class BaseAnsibleTaskCheck(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_modules: Iterable[str],
        block_type: str,
        guideline: str | None = None,
        path: str | None = None,
    ) -> None:
        supported_entities = [
            entity
            for module in supported_modules
            for entity in (f'[].tasks[?"{module}" != null][]', f'[?"{module}" != null][]')
        ]

        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type=block_type,
            guideline=guideline,
        )

        self.entity_conf: dict[str, Any]  # stores the complete entity configuration
        self.path = path
        self.supported_modules = supported_modules

        registry.register(self)

    def scan_entity_conf(  # type:ignore[override]  # multi_signature decorator is problematic
        self, conf: dict[str, Any], entity_type: str
    ) -> tuple[CheckResult, dict[str, Any]]:
        self.entity_type = entity_type
        self.entity_conf = conf

        module_conf = next((conf[module] for module in self.supported_modules if module in conf), None)
        if not module_conf:
            # this should actually never happen, but better to be safe, than sorry
            logging.info(f"Failed to find supported module {self.supported_modules} in {json.dumps(conf)}")
            return CheckResult.UNKNOWN, conf

        return self.scan_conf(module_conf)

    @abstractmethod
    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        pass



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AzureSearchSQLQueryUpdates(BaseResourceCheck):
    def __init__(self) -> None:
        # Cognitive Search services support indexing and querying. Indexing is the process of loading content
        # into the service to make it searchable. Querying is the process where a client searches for content
        # by sending queries to the index.
        # Cognitive Search supports a configurable number of replicas.
        # Having multiple replicas allows queries and index updates to load balance across multiple replicas.
        # To receive a Service Level Agreement (SLA) for Search index queries a minimum of 2 replicas is required.
        name = "Ensure that Azure Cognitive Search maintains SLA for search index queries"
        id = "CKV_AZURE_209"
        supported_resources = ("azurerm_search_service",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        self.evaluated_keys = ["replica_count"]

        replica_count = conf.get("replica_count")
        if replica_count and isinstance(replica_count, list):
            if replica_count[0] >= 2:
                return CheckResult.PASSED

        return CheckResult.FAILED


check = AzureSearchSQLQueryUpdates()
